import java.io.*;
import java.util.Arrays;


public class ContentEvolutionGeneration {
	
	
	/*
	 * ÿһʱε-ʾ
	 * topic-term matrix for each time slice.
	 */
	private TermAndFrequency[][] topic_term_matrix;
	
	private int numOfUniqueTerms;
	
	/*
	 * unique term 
	 */
	private String[] uniqueTerms;
	
	public void generateContentEvolution() {
		System.out.println("Topic Content Evolution generation start!");
		countNumOfUniqueTerms(new File("DocumentsAfterPreProcessing.vocab"));
		readUniqueTerms(new File("DocumentsAfterPreProcessing.vocab"));
		for(int k=0; k<ParamSetup.NUM_OF_TOPICS; k++) {
			String index;
			if(k<10)  index = "00" + k;
			else index = "0" + k;
			readProbability(new File("model_run\\lda-seq\\topic-"+ index +"-var-e-log-prob.dat"));
			sort();
			writeTopicTermMatrix(new File("ContentEvolution\\topic" + (k+1) + ".txt"));
		}
		System.out.println("Topic Content Evolution finished!");
		System.out.println("===============================================================");
	}
	
	public void countNumOfUniqueTerms(File f) {
		FileReader fr = null;
		BufferedReader br = null;
		int index = 0;
		String s = null;
		try {
			fr = new FileReader(f);
			br = new BufferedReader(fr);
			while((s = br.readLine()) != null) {
				index++;
			}
			numOfUniqueTerms = index;
			uniqueTerms = new String[numOfUniqueTerms];
			br.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	
	public void readUniqueTerms(File f) {
		FileReader fr = null;
		BufferedReader br = null;
		int index = 0;
		String s = null;
		try {
			fr = new FileReader(f);
			br = new BufferedReader(fr);
			while((s = br.readLine()) != null) {
				uniqueTerms[index++] = s;
			}
			br.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	public void readProbability(File f) {
		topic_term_matrix = new TermAndFrequency[ParamSetup.NUM_OF_TIME][numOfUniqueTerms];
		FileReader fr = null;
		BufferedReader br = null;
		String s = null;
		try {
			fr = new FileReader(f);
			br = new BufferedReader(fr);
			for(int i=0; i<numOfUniqueTerms; i++) {
				for(int j=0; j<ParamSetup.NUM_OF_TIME; j++) {
					s = br.readLine();
					String term = uniqueTerms[i];
					double frequency = Math.exp(Double.parseDouble(s));
					topic_term_matrix[j][i] = new TermAndFrequency(term, frequency);
				}
			}
			br.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	public void sort() {
		for(int i=0; i<topic_term_matrix.length; i++) {
			Arrays.sort(topic_term_matrix[i]);
		}
	}
	
	public void writeTopicTermMatrix(File f) {
		FileWriter fr;
		BufferedWriter br;
		try {
			fr = new FileWriter(f);
			br = new BufferedWriter(fr);
			for(int t=0; t<ParamSetup.NUM_OF_TIME; t++) {
				br.write((t+1) + ": ");
				for(int i=topic_term_matrix[t].length-1; i>=topic_term_matrix[t].length-10; i--) {
					br.write(topic_term_matrix[t][i].getWord() + " " + topic_term_matrix[t][i].getFrequency() + "  ");
				}
				br.newLine();
			}
			br.close();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new ContentEvolutionGeneration().generateContentEvolution();
	}
	
	private class TermAndFrequency implements Comparable<TermAndFrequency> {

		private String word;
		
		private double frequency;
		
		public TermAndFrequency(String word, double frequency) {
			this.word = word;
			this.frequency = frequency;
		}

		public String getWord() {
			return word;
		}

		public double getFrequency() {
			return frequency;
		}

		@Override
		public int compareTo(TermAndFrequency other) {
			return Double.compare(frequency, other.frequency);
		}
	}
}
