import java.util.*;
import java.io.*;

public class PreProcessing {
	
	public static final String INPUT_DIR = "DocumentsBeforePreProcessing";
	
	public static final String OUTPUT_DIR = "DocumentsAfterPreProcessing";
	
	private List<Document> documents = new ArrayList<Document>();
	
	private List<Word> wordList = new ArrayList<Word>();
	
	public void preProcess() {
		System.out.println("Preprocessing start!");
		tokenization();
		stopping();
		stemming();
		pruning();
		write();
		System.out.println("Preprocessing finished!");
		System.out.println("===============================================================");
	}
	
	public void tokenization() {
		System.out.println("start tokenization!");
		StringBuilder sb;
		FileReader fr;
		boolean tag;
		try {
			File[] documentFiles = new File(INPUT_DIR).listFiles();
			for(File documentFile : documentFiles) {
				tag = false;
				sb = new StringBuilder();
				fr = new FileReader(documentFile);
				Document document = new Document();
				int charFromFile = 0;
				while((charFromFile = fr.read()) != -1) {
					if((charFromFile >= 'a' && charFromFile <= 'z') || (charFromFile >= 'A' && charFromFile <= 'Z')) {
						sb.append((char)charFromFile);
						break;
					}
				}
			    while((charFromFile = fr.read()) != -1) {
					if((charFromFile >= 'a' && charFromFile <= 'z') || (charFromFile >= 'A' && charFromFile <= 'Z')) {
						if(tag == true) {
							document.addWord(sb.toString().toLowerCase());
							sb = new StringBuilder();
						}
						sb.append((char)charFromFile);
						tag = false;
					} else {
						tag = true;
					}
				}
			    document.addWord(sb.toString().toLowerCase());
				document.setName(documentFile.getName());
				documents.add(document);
				fr.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void stopping() {
		System.out.println("start stopping!");
		List<String> temp;
		String word;
		for(int i=0; i<documents.size(); i++) {
			temp = new ArrayList<String>();
			for(int j=0; j<documents.get(i).size(); j++) {
				word = documents.get(i).getWord(j);
				if(!Stopwords.isStopword(word)) {
					temp.add(word);
				}
			}
			documents.get(i).copy(temp);
		}
	}

	public void stemming() {
		System.out.println("start stemming!");
		List<String> temp;
		String word;
		Stemmer stm;
		for(int i=0; i<documents.size(); i++) {
			temp = new ArrayList<String>();
			for(int j=0; j<documents.get(i).size(); j++) {
				word = documents.get(i).getWord(j);
				char[] chars = word.toCharArray();
				stm = new Stemmer();
				for(int k=0; k<chars.length; k++) {
					stm.add(chars[k]);
				}
				stm.stem();
				temp.add(stm.toString());
			}
			documents.get(i).copy(temp);
		}
	}
	
	public void pruning() {
		System.out.println("start pruning!");
		List<String> temp;
		countWordFenquency();
		markWordToBePruned();
		for(int i=0; i<documents.size(); i++) {
			temp = new ArrayList<String>();
			for(int j=0; j<documents.get(i).size(); j++) {
				Word word = new Word(documents.get(i).getWord(j));
				Iterator<Word> iter = wordList.iterator();
				while(iter.hasNext()) {
					Word temp2 = iter.next();
					if(temp2.equals(word)) {
						if(!temp2.isWordToRemove()) {
							temp.add(temp2.getWord());
							break;
						}
					}
				}
			}
			documents.get(i).copy(temp);
		}
	}
	
	private void countWordFenquency() {
		String word;
		for(int i=0; i<documents.size(); i++) {
			for(int j=0; j<documents.get(i).size(); j++) {
				word = documents.get(i).getWord(j);
				Iterator<Word> itr = wordList.iterator();
				boolean tag = false;
				while(itr.hasNext())
				{
					Word temp = itr.next();
					if(temp.equals(new Word(word)))
					{
						temp.addCount();
						tag = true;
						break;
					}
				}
				if(tag == false) wordList.add(new Word(word));
			}
		}
	}
	
	private void markWordToBePruned() {
		Iterator<Word> itr = wordList.iterator();
		while(itr.hasNext()) {
			Word temp = itr.next();
			if(temp.getCount() <= ParamSetup.PRUNE_THRESHOLD) {
				temp.setTag(true);
			}
		}
	}
	
	public void write() {
		FileWriter fw;
		BufferedWriter bw;
		for(int i=0; i<documents.size(); i++) {
			try {
				fw = new FileWriter(new File(OUTPUT_DIR + "\\" + documents.get(i).getName()));
				bw = new BufferedWriter(fw);
				for(int j=0; j<documents.get(i).size(); j++) {
					bw.write(documents.get(i).getWord(j));
					bw.newLine();
				}
				bw.close();
			} catch(IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	private class Document {
		private List<String> words = new ArrayList<String>();
		private String name;
		public void addWord(String word) {
			words.add(word);
		}
		public String getWord(int index) {
			return words.get(index);
		}
		public void copy(List<String> temp) {
			words.clear();
			words.addAll(temp);
		}
		public int size() {
			return words.size();
		}
		public void setName(String name) {
			this.name = name;
		}
		public String getName() {
			return name;
		}
	}
	
	private class Word
	{
		private String word;//the name of the word
		private int count;//the appearing times of this word
		private boolean tag;//whether this word should be removed from corpus
		
		public Word(String word)
		{
			this.word = word;
			count = 1;
			tag = false;
		}
		
		public String getWord() {return word;}
		
		public int getCount() {return count;}
		
		public boolean isWordToRemove() {return tag;}
		
		public void setTag(boolean tag) {this.tag = tag;}
		
		public boolean equals(Object other)
		{
			Word o = (Word)other;
			if(word.equals(o.word)) return true;
			else return false;
		}
		
		public void addCount() {count++;}
	}

}
