import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;


public class StrengthEvolutionGeneration {
	
	private int numOfDocs;
	
	//¼ÿʱεĵ
	private int[] num_of_docs_in_each_time = new int[ParamSetup.NUM_OF_TIME];
	
	//ĵ
	private double[][] doc_topic_matrix;
	
	//ʱ仯assignmentֵ
	private double[] assignment_of_a_topic;
	
	public void generateStrengthEvolution() {
		System.out.println("Topic Strength evolution generation start!");
		countNumOfDocs(new File("DocumentsAfterPreProcessing.dmap"));
		readNumOfDocsInEachTime(new File("test-seq.dat"));
		readProbability(new File("model_run\\lda-seq\\gam.dat"));
		countProbability();
		countAndWriteAssignment();
		System.out.println("Topic Strength evolution finished!");
		System.out.println("===============================================================");
	}
	
	public void countNumOfDocs(File f) {
		int count = 0;
		FileReader fr;
		BufferedReader br;
		try {
			fr = new FileReader(f);
			br = new BufferedReader(fr);
			String s;
			while((s = br.readLine()) != null) {
				count++;
			}
			numOfDocs = count;
			br.close();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}

	public void readNumOfDocsInEachTime(File f) {
		FileReader fr = null;
		BufferedReader br = null;
		try {
			fr = new FileReader(f);
			br = new BufferedReader(fr);
			String s = br.readLine();
			int index = 0;
			while((s = br.readLine()) != null) {
				num_of_docs_in_each_time[index++] = Integer.parseInt(s);
			}
			br.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	public void readProbability(File f) {
		doc_topic_matrix = new double[numOfDocs][ParamSetup.NUM_OF_TOPICS];
		FileReader fr = null;
		BufferedReader br = null;
		try {
			fr = new FileReader(f);
			br = new BufferedReader(fr);
			for(int i=0; i<numOfDocs; i++) {
				for(int j=0; j<ParamSetup.NUM_OF_TOPICS; j++) {
					doc_topic_matrix[i][j] = Double.parseDouble(br.readLine());
				}
			}
			br.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
	
	public void countProbability() {
		for(int i=0; i<numOfDocs; i++) {
			double sum = 0;
			for(int j=0; j<ParamSetup.NUM_OF_TOPICS; j++) {
				sum += doc_topic_matrix[i][j];
			}
			for(int j=0; j<ParamSetup.NUM_OF_TOPICS; j++) {
				doc_topic_matrix[i][j] /= sum;
			}
		}
	}
	
	public void countAndWriteAssignment() {
		for(int i=0; i<ParamSetup.NUM_OF_TOPICS; i++) {
			countAssignment(i);
			writeAssignment(new File("StrengthEvolution\\" + (i+1) + ".txt"));
		}
	}
	
	
	/*
	 * k starts from 0
	 */
	private void countAssignment(int k) {
		assignment_of_a_topic = new double[ParamSetup.NUM_OF_TIME];
		int docCount = 0;
		for(int i=0; i<ParamSetup.NUM_OF_TIME; i++) {
			double assignment = 0;
			for(int j=0; j<num_of_docs_in_each_time[i]; j++) {
				assignment += doc_topic_matrix[docCount][k];
				docCount++;
			}
			assignment_of_a_topic[i] = assignment / num_of_docs_in_each_time[i];
		}
	}
	
	private void writeAssignment(File f) {
		try {
			FileWriter fw = new FileWriter(f);
			BufferedWriter bw = new BufferedWriter(fw);
			for(int i=0; i<assignment_of_a_topic.length; i++) {
				bw.write(assignment_of_a_topic[i]+"");
				bw.newLine();
			}
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	

	public static void main(String[] args) {
		new StrengthEvolutionGeneration().generateStrengthEvolution();
	}

}
